package com.contextspace;

import static org.junit.Assert.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.FetchType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;

import org.junit.Test;

public class TestOneToMany {

    @Test
    public void test() {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("objectdb:$objectdb/db/test.tmp;drop");
        EntityManager em = emf.createEntityManager();
        
        Team teamMovieStars = new Team();
        teamMovieStars.setName("Team Movie Stars");
        teamMovieStars.addPlayer(new Player("Brad Pitt"));
        teamMovieStars.addPlayer(new Player("Angelina Jolie"));
        
        em.getTransaction().begin();
        em.persist(teamMovieStars);
        em.getTransaction().commit();
        em.flush();
        
        TypedQuery<Long> query = em.createQuery("SELECT count(c) FROM Player c", Long.class);
        Long cdiCount = query.getSingleResult();
        assertEquals(2, cdiCount.intValue());
        
        TypedQuery<Player> query2 = em.createQuery("SELECT c FROM Player c where c.name='Brad Pitt'", Player.class);
        List<Player> results = query2.getResultList();
        assertEquals(1, results.size());
        assertEquals("Brad Pitt", results.get(0).getName());
        assertNotNull(results.get(0).getTeam());
        assertEquals("Team Movie Stars", results.get(0).getTeam().getName());
        
        em.close();
        emf.close();
    }

    @Entity
    public static class Team implements Serializable {

        private static final long serialVersionUID = 1L;
        
        private List<Player> squad = new ArrayList<Player>();
        private String name = null;

        @OneToMany(targetEntity = Player.class, mappedBy = "team", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
        public List<Player> getSquad() {
            return squad;
        }

        /**
         * @param squad the squad to set
         */
        public void setSquad(List<Player> squad) {
            this.squad = squad;
        }
        
        public void addPlayer(Player player) {
            this.squad.add(player);
        }

        /**
         * @return the name
         */
        public String getName() {
            return name;
        }

        /**
         * @param name the name to set
         */
        public void setName(String name) {
            this.name = name;
        }
    }

    @Entity
    public static class Player implements Serializable {

        private static final long serialVersionUID = 1L;
 
        private Team team;
        private String name = null;

        /**
         * Default Constructor
         */
        public Player() {
            
        }
        
        /**
         * Constructor
         * @param name
         */
        public Player(String name) {
            this.name = name;
        }
        
        @ManyToOne
        public Team getTeam() {
            return team;
        }

        /**
         * @return the name
         */
        public String getName() {
            return name;
        }

        /**
         * @param name the name to set
         */
        public void setName(String name) {
            this.name = name;
        }  
    }

}
